import { getNameOfDescriptor } from "../helpers/getNameOfDescriptor.js";
export class ServiceInstanceInfo {
    descriptor;
    get instance() {
        if (!this.initialized) {
            throw new Error(`Service "${getNameOfDescriptor(this.descriptor)}" is not initialized (circular dependency)`);
        }
        return this._instance;
    }
    initialized;
    disposed;
    dependents;
    _instance;
    constructor(descriptor) {
        this.descriptor = descriptor;
        this.initialized = false;
        this.disposed = false;
        this.dependents = new Set();
    }
    addInstance(instance) {
        if (this.initialized) {
            throw new Error("Instance already initialized");
        }
        this._instance = instance;
        this.initialized = true;
    }
    addDependent(dependent) {
        this.dependents.add(dependent);
    }
    disposeSync() {
        const instance = this.instance;
        if (typeof instance[Symbol.dispose] === "function" &&
            !this.disposed &&
            this.initialized) {
            for (const dependent of this.dependents) {
                dependent.disposeSync();
            }
            instance[Symbol.dispose]();
            this.disposed = true;
        }
    }
    async disposeAsync() {
        const instance = this.instance;
        if (!this.disposed && this.initialized) {
            if (typeof instance[Symbol.asyncDispose] === "function") {
                this.disposed = true;
                try {
                    await Promise.all([...this.dependents].map((dependent) => dependent.disposeAsync()));
                    await instance[Symbol.asyncDispose]();
                }
                catch (e) {
                    this.disposed = false;
                    throw e;
                }
            }
            else {
                this.disposeSync();
            }
        }
    }
}
//# sourceMappingURL=ServiceInstanceInfo.js.map