export class ModuleRegistry {
    static [Symbol.iterator]() {
        return this.modules.values();
    }
    static modules = new Map();
    static listeners = [];
    static add(module) {
        if (this.has(module.name)) {
            throw new Error(`Module ${module.name} is already added.`);
        }
        this.modules.set(module.name, module);
        this.listeners.forEach((listener) => listener(module));
    }
    static has(name) {
        return this.modules.has(name);
    }
    static get(name) {
        return this.modules.get(name);
    }
    static addListener(listener) {
        this.listeners.push(listener);
    }
}
//# sourceMappingURL=ModuleRegistry.js.map